// Copyright (C) Univ. Paris-SUD, Johan Oudinet <oudinet@lri.fr> - 2009
//  
// This file is part of Rukia.
//  
// Rukia is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.
//  
// Rukia is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//  
// You should have received a copy of the GNU Lesser General Public
// License along with Rukia.  If not, see <http://www.gnu.org/licenses/>.
//  
#ifndef RUKIA_DRAW_HXX
# define RUKIA_DRAW_HXX
# include <iostream>
# include <boost/graph/iteration_macros.hpp>
# include "draw.hh"

namespace rukia {
  //
  // Recursive method
  //
  template <class Automaton, class Ctable, class Random>
  draw_paths<Automaton,Ctable,Random>::draw_paths (const Automaton& aut,
						   const Ctable& ctable,
						   Random& ralgo)
    : aut_(aut), ctable_(ctable), ralgo_(ralgo)
  {
  }

  // TODO: vertex_descriptor s should be an argument.
  template <class Automaton, class Ctable, class Random>
  path_type
  draw_paths<Automaton,Ctable,Random>::operator()(unsigned n) const
  {
    typedef typename Ctable::value_type			value_type;

    path_type path (n);
    vertex_descriptor s = vertex (0, aut_);
    unsigned i = n;
    if (ctable_(s,n) == 0)
      {
	std::cerr << "There is no path starting from " << aut_[s].id
		  << " of length " << n << "!" << std::endl;
	exit(2);
      }
    for (path_type::iterator it = path.begin ();
	 it != path.end (); ++it, --i)
      {
	value_type acc = 0;
	value_type r;
	r = ralgo_(ctable_ (s, i));
	BGL_FORALL_OUTEDGES_T(s, e, aut_, Automaton)
	  {
	    acc += ctable_ (target (e, aut_), i-1);
	    if (acc > r)
	      {
		*it = e;
		s = target (e, aut_);
		break;
	      }
	  }
      }
    return path;
  }

  //
  // Boltzmann generator
  //
  template <class Automaton, class Coeffs, class Random>
  boltzmann_generator<Automaton,Coeffs,Random>
  ::boltzmann_generator (const Automaton& aut,
			 const Coeffs& coeffs,
			 const value_type& z,
			 Random& ralgo)
    : aut_(aut), coeffs_(coeffs), z_(z), ralgo_(ralgo)
  {
  }

  // TODO: vertex_descriptor s should be an argument.
  template <class Automaton, class Coeffs, class Random>
  path_type
  boltzmann_generator<Automaton,Coeffs,Random>::operator()(unsigned n) const
  {
    path_type path (n);
    vertex_descriptor s = vertex (0, aut_);
    unsigned i;
    bool over;
    path_type::iterator path_iter;
    if (out_degree(s, aut_) == 0 || coeffs_[aut_[s].id] == 0)
      {
	std::cerr << "There is no path starting from " << aut_[s].id
		  << " of length " << n << "!" << std::endl;
	exit(2);
      }
    do {
      s = vertex (0, aut_);
      i = 0;
      over = false;
      path_iter = path.begin ();
      while (!over)
	{
	  over = true;
	  value_type acc = 0;
	  value_type r;
	  r = ralgo_(coeffs_[aut_[s].id]);
	  BGL_FORALL_OUTEDGES_T(s, e, aut_, Automaton)
	    {
	      acc += z_ * coeffs_[aut_[target (e, aut_)].id];
	      if (acc > r)
		{
		  ++i;
		  if (path_iter == path.end ())
		    break; // path too long, abort.
		  over = false;
		  *path_iter++ = e;
		  s = target (e, aut_);
		  break;
		}
	    }
	}
    } while (i != n); // path too short, restart.
    return path;
  }

  //
  // Random Walks
  //
  template <class Automaton, class Random>
  random_walks<Automaton,Random>::random_walks (const Automaton& aut,
						Random& ralgo)
    : aut_(aut), ralgo_(ralgo)
  {
  }

  // TODO: vertex_descriptor s should be an argument.
  template <class Automaton, class Random>
  path_type
  random_walks<Automaton,Random>::operator()(unsigned n) const
  {
    typedef typename boost::graph_traits<Automaton>
      ::degree_size_type value_type;

    path_type path (n);
    vertex_descriptor s = vertex (0, aut_);
    unsigned i;
    if (out_degree(s, aut_) == 0)
      {
	std::cerr << "There is no path starting from " << aut_[s].id
		  << " of length " << n << "!" << std::endl;
	exit(2);
      }
    do {
      i = n;
      for (path_type::iterator it = path.begin ();
	   it != path.end (); ++it, --i)
	{
	  value_type acc = 0;
	  value_type r;
	  if (out_degree(s, aut_) == 0)
	    {
	      std::cerr << "Got a deadlock before reach the end of path. Retrying..."
			<< std::endl;
	      break;
	    }
	  r = ralgo_(out_degree (s, aut_));
	  BGL_FORALL_OUTEDGES_T(s, e, aut_, Automaton)
	    {
	      ++acc;
	      if (acc > r)
		{
		  *it = e;
		  s = target (e, aut_);
		  break;
		}
	    }
	}
    } while (i != 0);
    return path;
  }


} // end namespace rukia

#endif // ! RUKIA_DRAW_HXX
